@@ -64,8 +64,10 @@ class CustomConfiguration(object):
         self.__button_ok            = self.__ui.get_widget('button_ok')
         self.__button_upload        = self.__ui.get_widget('button_upload')
         self.__button_detect_basics = self.__ui.get_widget('button_detect_basics')
+        self.__button_keys_learn    = self.__ui.get_widget('button_keys_learn')
         self.__button_keys_remove   = self.__ui.get_widget('button_keys_remove')
         self.__button_keys_clear    = self.__ui.get_widget('button_keys_clear')
+        self.__button_keys_add      = self.__ui.get_widget('button_keys_add')
 
         # setup model page:
         self.__entry_vendor      = self.__ui.get_widget('entry_vendor')
@@ -84,6 +86,11 @@ class CustomConfiguration(object):
         # apply "secondary" child property: for some reason libglade ignores it
         self.__dialog.action_area.set_child_secondary(self.__button_upload, True)
 
+        # apply stock icon to learn button
+        image = gtk.image_new_from_stock(gtk.STOCK_MEDIA_RECORD,
+                                         gtk.ICON_SIZE_BUTTON)
+        self.__button_keys_learn.set_image(image)
+
     # pylint: disable-msg=W0201
     def __setup_basics(self):
         '''Initialize widgets of the "Basics" page.'''
@@ -110,6 +117,13 @@ class CustomConfiguration(object):
         self.__page_keys       = self.__ui.get_widget('page_keys')
         self.__treeview_keys   = self.__ui.get_widget('treeview_keys')
         self.__hbuttonbox_keys = self.__ui.get_widget('hbuttonbox_keys')
+        self.__label_keys_hint = self.__ui.get_widget('label_keys_hint')
+        self.__image_keys_hint = self.__ui.get_widget('image_keys_hint')
+
+        self.__keys_learning_hint = _(
+            '<b>Learning new key code.</b>\n'
+            'Press the button on your remote control which should emit this key-code.')
+        self.__keys_default_hint = self.__label_keys_hint.get_label()
 
         # pylint: disable-msg=W0613
         def editing_started(cell, editable, path):
@@ -140,6 +154,8 @@ class CustomConfiguration(object):
                 keys_model.update_key(new_name, mapping.code)
                 self.select_key(new_name)
 
+            self.__treeview_keys.grab_focus()
+
         keys_renderer = gtk.CellRendererCombo()
         keys_renderer.set_properties(editable=True, text_column=0,
                                      model=self.default_keys)
@@ -261,6 +277,7 @@ class CustomConfiguration(object):
         device = self.__device or ''
 
         # pylint: disable-msg=W0613
+        @thread_callback
         def report_success(result, sender=None):
             '''Handle success reported by the service backend.'''
 
@@ -275,6 +292,7 @@ class CustomConfiguration(object):
                 self.__treeview_keys.get_model().update(remote.key_codes)
 
         # pylint: disable-msg=W0613
+        @thread_callback
         def report_failure(message, sender=None):
             '''Handle failures reported by the service backend.'''
 
@@ -286,8 +304,15 @@ class CustomConfiguration(object):
         try:
             bus     = backend.get_service_bus()
             service = backend.get_service(bus)
+
+            # Stop the lirc deamon, as some drivers (e.g. lirc_streamzap) do
+            # not support concurrent device access:
+            service.ManageLircDaemon('stop')
+
             driver  = service.LearnKeyCode(driver, device, configuration, [mapping.key])
-            driver  = bus.get_object(service.requested_bus_name, driver)
+
+            driver = bus.get_object(service.requested_bus_name, driver)
+            driver = dbus.Interface(driver, backend.ExternalToolDriver.INTERFACE_NAME)
 
             self.__learning_driver = driver
 
@@ -297,7 +322,9 @@ class CustomConfiguration(object):
             driver.Execute()
 
         except dbus.DBusException, ex:
-            report_failure(ex.message)
+            report_failure.callback(ex.message)
+
+        self._on_dialog_changed()
 
     def __stop_learning(self):
         '''Stop learning of a remote control's key code.'''
@@ -309,6 +336,13 @@ class CustomConfiguration(object):
             except dbus.DBusException, ex:
                 logging.error(ex)
 
+            try:
+                # restart the lirc deamon (we stopped it before learning):
+                backend.get_service().ManageLircDaemon('start')
+
+            except dbus.DBusException, ex:
+                logging.error(ex)
+
             self.__learning_driver = None
 
         if self.__learning_row:
@@ -316,9 +350,12 @@ class CustomConfiguration(object):
             keys.set_state(self.__learning_row.get_path(), None)
             self.__learning_row = None
 
+        self._on_dialog_changed()
+
     # pylint: disable-msg=C0103,W0613
     def _on_treeview_keys_selection_changed(self, selection):
         '''Handle selection of row in the "keys" tree view.'''
+        self._on_dialog_changed()
         self.__stop_learning()
 
     # pylint: disable-msg=W0613
@@ -376,7 +413,7 @@ class CustomConfiguration(object):
         self.__dialog.set_sensitive(False)
 
         net.post_file(target_uri=config.URI_UPLOADS,
-                      filename=lirc.find_remote_config(),
+                      filename=lirc.find_remote_config()[0],
                       content=self.__remote.configuration,
                       context=_('customized configuration file'),
                       finished_callback=on_upload_finished,
@@ -438,11 +475,19 @@ class CustomConfiguration(object):
             self.__progressbar_detect_basics.set_fraction(0)
 
             if details:
+                # TODO: This dialog should probably have a cancel button.
+                response_buttons = (
+                    (gtk.RESPONSE_ACCEPT, _('_Start'), gtk.STOCK_EXECUTE),
+                )
+
                 show_message(self.__dialog, message, details,
-                             message_type=gtk.MESSAGE_INFO)
+                             message_type=gtk.MESSAGE_INFO,
+                             buttons=response_buttons)
 
             driver.Proceed()
 
+        bus, service, driver = None, None, None
+
         try:
             self.__stop_detection()
 
@@ -451,7 +496,9 @@ class CustomConfiguration(object):
 
             driver = service.DetectParameters(self.__receiver.lirc_driver or '',
                                               self.__device or '')
+
             driver = bus.get_object(service.requested_bus_name, driver)
+            driver = dbus.Interface(driver, backend.ExternalToolDriver.INTERFACE_NAME)
 
             driver.connect_to_signal('RequestAction',  request_action)
             driver.connect_to_signal('ReportProgress', report_progress)
@@ -467,7 +514,7 @@ class CustomConfiguration(object):
             self.__progressbar_detect_basics.show()
 
         except dbus.DBusException, ex:
-            report_failure(ex.message)
+            report_failure.callback(ex.message)
             self.__stop_detection()
 
     def __stop_detection(self):
@@ -489,11 +536,15 @@ class CustomConfiguration(object):
         have_receiver = (None != self.__receiver)
         have_basics = bool(self.__treeview_basics.get_model())
         have_keys = bool(self.__treeview_keys.get_model())
+        is_learning = self.__learning_row is not None
 
         model_complete = (bool(self.vendor_name) and bool(self.product_name))
         basics_complete = (model_complete and have_basics)
         keys_complete = (basics_complete and have_keys)
 
+        selection = self.__treeview_keys.get_selection()
+        keys_iter = selection.get_selected()[1]
+
         # Update state of global widgets:
         self.__button_ok.set_sensitive(model_complete)
 
@@ -508,8 +559,18 @@ class CustomConfiguration(object):
         # Update state of keys-page widgets:
         self.__treeview_keys.set_sensitive(basics_complete and have_receiver)
         self.__hbuttonbox_keys.set_sensitive(basics_complete and have_receiver)
-        self.__button_keys_remove.set_sensitive(have_keys)
-        self.__button_keys_clear.set_sensitive(have_keys)
+
+        self.__button_keys_add.set_sensitive(not is_learning)
+        self.__button_keys_remove.set_sensitive(have_keys and not is_learning)
+        self.__button_keys_clear.set_sensitive(have_keys and not is_learning)
+        self.__button_keys_learn.set_sensitive(keys_iter is not None)
+        self.__button_keys_learn.set_active(is_learning)
+
+        self.__label_keys_hint.set_markup(is_learning and
+                                          self.__keys_learning_hint or
+                                          self.__keys_default_hint)
+
+        self.__image_keys_hint.set_property('visible', is_learning)
 
     def select_key(self, key_name):
         '''Select the row of the specified key.'''
@@ -596,6 +657,18 @@ class CustomConfiguration(object):
         selection.select_path(path)
         self._on_dialog_changed()
 
+    def _on_button_keys_learn_toggled(self, button):
+        '''Handle the "Learn Key Code" button's "clicked" signal.'''
+
+        if self.__button_keys_learn.get_active():
+            selection = self.__treeview_keys.get_selection()
+            keys_model, keys_iter = selection.get_selected()
+            path = keys_model.get_path(keys_iter)
+            self.__start_learning(path)
+
+        else:
+            self.__stop_learning()
+
     def _on_button_keys_clear_clicked(self, button):
         '''Handle the "Clear" button's "clicked" signal.'''
 
