@@ -19,7 +19,7 @@
 The main window of the application.
 '''
 
-import dbus, errno, gobject, gtk, gtk.gdk, gtk.glade
+import dbus, errno, gobject, gtk, gtk.gdk, gtk.glade, pango
 import httplib, locale, logging, os, subprocess
 
 from gettext               import gettext as _
@@ -52,7 +52,6 @@ class RemoteControlProperties(object):
         self.__setup_size_groups()
 
         # Look in the configuration to show previously-chosen details:
-
         self.__restore_hardware_settings()
 
         # Allow UI changes from being written back to configuration files:
@@ -71,7 +70,7 @@ class RemoteControlProperties(object):
         self.__hardware_manager.connect('search-finished', self._on_search_finished)
         self.__hardware_manager.connect('receiver-found',  self._on_receiver_found)
 
-        self.__receiver_vendors = model.ReceiverVendorList()
+        self.__receiver_vendors = model.ReceiverVendorList(self.__hardware_manager)
         self.__receiver_vendors.load(receivers_db)
 
         self.__remote_vendors = model.RemoteVendorList()
@@ -110,6 +109,10 @@ class RemoteControlProperties(object):
     def __lookup_widgets(self):
         '''Initialize widget attributes from Glade file.'''
 
+        # This method is more robust than looking up and assigning widgets
+        # manually, but it also completly screws up pychecker and pylint. For
+        # pylint we ship a plugin to fix this.
+
         # pylint: disable-msg=W0201
 
         widget_list = (
@@ -124,7 +127,7 @@ class RemoteControlProperties(object):
 
             'label_device',
             'combo_device',
-            'entry_device',
+            'label_device_name',
             'spinbutton_device',
 
             # remote widgets:
@@ -137,7 +140,6 @@ class RemoteControlProperties(object):
             'button_download',
 
             # preview widgets:
-            'image_preview_status',
             'label_preview_status',
             'label_preview_result',
         )
@@ -149,6 +151,8 @@ class RemoteControlProperties(object):
             setattr(self, attr, widget)
 
         self.__dialog = self.__ui.get_widget('lirc_properties_dialog')
+        self.__entry_device = self.__combo_device.get_child()
+        self.__table_receiver_selection.set_row_spacing(2, 0)
 
     def __setup_size_groups(self):
         '''
@@ -165,14 +169,20 @@ class RemoteControlProperties(object):
         size_group.add_widget(self.__alignment_auto_detect)
 
         size_group = gtk.SizeGroup(gtk.SIZE_GROUP_VERTICAL)
-        size_group.add_widget(self.__image_preview_status)
         size_group.add_widget(self.__label_preview_status)
         size_group.add_widget(self.__label_preview_result)
 
     def __setup_product_lists(self):
         '''Initialize widgets with product listings.'''
 
-        self.__combo_receiver_vendor_list.set_model(self.__receiver_vendors)
+        def vendor_has_products(model, iter):
+            products, = model.get(iter, 1)
+            return len(products) > 0
+
+        vendor_list = self.__receiver_vendors.filter_new()
+        vendor_list.set_visible_func(vendor_has_products)
+
+        self.__combo_receiver_vendor_list.set_model(vendor_list)
         self.__combo_receiver_vendor_list.set_active(0)
 
         self.__combo_remote_vendor_list.set_model(self.__remote_vendors)
@@ -181,6 +191,12 @@ class RemoteControlProperties(object):
         # update widget sensitivity
         self._on_radiobutton_supplied_remote_toggled()
 
+        # apply ellipses to combo boxes
+        for widget in (
+            self.__combo_receiver_vendor_list, self.__combo_receiver_product_list,
+            self.__combo_remote_vendor_list, self.__combo_remote_product_list):
+            widget.get_cells()[0].set_property('ellipsize', pango.ELLIPSIZE_END)
+
     def __setup_authorization(self):
         '''Initialize authorization facilities.'''
 
@@ -202,10 +218,16 @@ class RemoteControlProperties(object):
         settings = lirc.HardwareConfParser(config.LIRC_HARDWARE_CONF)
 
         # Try to select configured receiver vendor:
+        #
+        # NOTE: Ubuntu's lirc script doesn't distinguish between remote and
+        # receiver settings in "hardware.conf", whereas we have to. For that
+        # reason the device node's name is stored in REMOTE_DEVICE, instead
+        # of RECEIVER_DEVICE.
+        #
         self.selected_receiver = (
             settings.get('RECEIVER_VENDOR'),
             settings.get('RECEIVER_MODEL'),
-            settings.get('RECEIVER_DEVICE'),
+            settings.get('REMOTE_DEVICE'),
         )
 
         # Try to select configured remote vendor:
@@ -224,14 +246,11 @@ class RemoteControlProperties(object):
             self.__label_preview_result.show()
             self.__label_preview_result.set_text(_('<none>'))
             self.__label_preview_status.set_text(_('Press remote control buttons to test:'))
-            self.__image_preview_status.hide()
 
         else:
             self.__label_preview_result.hide()
-            self.__label_preview_status.set_text(_('Remote control daemon not running. ' +
-                                                   'Cannot test buttons.'))
-            self.__image_preview_status.show()
-
+            self.__label_preview_status.set_markup(_('<b>Warning:</b> Remote control daemon ' +
+                                                     'not running. Cannot test buttons.'))
 
     # pylint: disable-msg=W0613,R0913
     def __on_lirc_key_pressed(self, listener, remote, repeat, name, code):
@@ -362,6 +381,11 @@ class RemoteControlProperties(object):
         '''
 
         tree_iter = vendor_list.get_active_iter()
+
+        if not tree_iter:
+            vendor_list.set_active(0)
+            return
+
         vendors = vendor_list.get_model()
         products, = vendors.get(tree_iter, 1)
 
@@ -380,17 +404,32 @@ class RemoteControlProperties(object):
             # populate device combo's item list
             device_node_model = self.__combo_device.get_model()
 
-            if not device_node_model:
-                device_node_model = gtk.ListStore(gobject.TYPE_STRING)
+            if device_node_model is None:
+                device_node_model = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
+                device_node_model.set_sort_column_id(0, gtk.SORT_ASCENDING)
                 self.__combo_device.set_model(device_node_model)
                 self.__combo_device.set_text_column(0)
 
+                self.__combo_device.clear()
+
+                renderer = gtk.CellRendererText()
+                self.__combo_device.pack_start(renderer, True)
+                self.__combo_device.set_attributes(renderer, markup=2)
+
             else:
                 device_node_model.clear()
 
-            for node in device_nodes:
+            for device in device_nodes:
+                if isinstance(device, tuple):
+                    args = map(gobject.markup_escape_text, reversed(device))
+                    markup = '<b>%s</b>\n<small>%s</small>' % tuple(args)
+                    name, device = device
+
+                else:
+                    name, markup = '', gobject.markup_escape_text(device)
+
                 tree_iter = device_node_model.append()
-                device_node_model.set(tree_iter, 0, node)
+                device_node_model.set(tree_iter, 0, device, 1, name, 2, markup)
 
             # activate first device node
             self.__combo_device.set_sensitive(True)
@@ -399,7 +438,7 @@ class RemoteControlProperties(object):
         else:
             # deactivate combo box, if device is not configurable
             self.__combo_device.set_sensitive(False)
-            self.selected_device = ''
+            self.selected_device = None
 
 
     def __setup_numeric_device(self, device_node):
@@ -429,6 +468,16 @@ class RemoteControlProperties(object):
 
         self.selected_receiver = receiver, device
 
+        tree_model = self.__combo_device.get_model()
+        tree_iter = self.__combo_device.get_active_iter()
+
+        device_name = (
+            tree_model is not None and tree_iter is not None and
+            tree_model.get(tree_iter, 1)[0] or '')
+
+        markup = gobject.markup_escape_text(device_name)
+        self.__label_device_name.set_markup('<small>%s</small>' % markup)
+
     # pylint: disable-msg=C0103,W0613
     def _on_spinbutton_device_value_changed(self, spinbutton=None):
         '''Handle changes to the spin-button for numeric device selection.'''
@@ -440,6 +489,11 @@ class RemoteControlProperties(object):
 
         # lookup selection:
         tree_iter = product_list.get_active_iter()
+
+        if tree_iter is None:
+            product_list.set_active(0)
+            return
+
         receiver, = product_list.get_model().get(tree_iter, 1)
 
         #  freeze configuration updates:
@@ -447,13 +501,15 @@ class RemoteControlProperties(object):
 
         try:
             # resolve device nodes:
-            device_nodes = receiver and receiver.device_nodes or None
+            device_nodes = (receiver and receiver.device_nodes or [])
 
             if device_nodes:
                 device_nodes = [node.strip() for node in device_nodes.split(',')]
                 device_nodes = self.__hardware_manager.resolve_device_nodes(device_nodes)
 
-            if 1 == len(device_nodes or []) and device_nodes[0].startswith('numeric:'):
+            if (1 == len(device_nodes) and
+                isinstance(device_nodes[0], str) and
+                device_nodes[0].startswith('numeric:')):
                 # show some spinbutton, when the device node requires numeric input
                 self.__setup_numeric_device(device_nodes[0])
 
@@ -500,7 +556,7 @@ class RemoteControlProperties(object):
             self.__progressbar_auto_detect.set_fraction(fraction)
 
         # gtk.ProgressBar doesn't support any text padding.
-        # Work arround that visual glitch with spaces.
+        # Work around that visual glitch with spaces.
         self.__progressbar_auto_detect.set_text(' %s ' % message)
 
     def _on_search_finished(self, manager = None):
@@ -516,8 +572,7 @@ class RemoteControlProperties(object):
             self._select_chosen_receiver()
 
         elif 0 == n_receivers:
-            tree_iter = self.__receiver_vendors.get_iter_first()
-            self.__combo_receiver_vendor_list.set_active_iter(tree_iter)
+            self.__combo_receiver_vendor_list.set_active(0)
 
             responses = (
                 (gtk.RESPONSE_CANCEL, gtk.STOCK_CANCEL),
@@ -625,11 +680,6 @@ class RemoteControlProperties(object):
         except dbus.DBusException, ex:
             logging.error(ex)
 
-    def __write_receiver_configuration(self, service, receiver, device):
-        pass
-    def __write_remote_configuration(self, service, remote):
-        pass
-
     def _on_receiver_chooser_dialog_response(self, dialog, response):
         '''Handle confirmed selections in the chooser for auto-detected receivers.'''
 
@@ -799,6 +849,7 @@ class RemoteControlProperties(object):
 
         vendor_name, product_name, device = None, None, None
 
+        # unpack the argument(s) passed:
         if isinstance(receiver, (tuple, list)):
             if 3 == len(receiver):
                 vendor_name, product_name, device = receiver
@@ -823,7 +874,9 @@ class RemoteControlProperties(object):
             products = None
 
             if tree_iter:
-                self.__combo_receiver_vendor_list.set_active_iter(tree_iter)
+                filter = self.__combo_receiver_vendor_list.get_model()
+                filter_iter = filter.convert_child_iter_to_iter(tree_iter)
+                self.__combo_receiver_vendor_list.set_active_iter(filter_iter)
                 products, = self.__receiver_vendors.get(tree_iter, 1)
 
             # highlight the selected receiver's product name:
@@ -833,7 +886,7 @@ class RemoteControlProperties(object):
                 self.__combo_receiver_product_list.set_active_iter(tree_iter)
 
             # enter the selected receiver's device node:
-            self.selected_device = (device or '')
+            self.selected_device = device
 
         finally:
             self._end_update_configuration(receiver, device)
@@ -841,7 +894,6 @@ class RemoteControlProperties(object):
     def __get_selected_remote(self):
         '''Retrieve the currently selected remote.'''
 
-        # TODO: gnome_lirc_properties/ui/RemoteControlProperties.py:844: No class attribute (_RemoteControlProperties__combo_remote_product_list) found
         tree_iter = self.__combo_remote_product_list.get_active_iter()
         remote = None
 
@@ -902,7 +954,36 @@ class RemoteControlProperties(object):
 
     def __set_selected_device(self, device_node):
         '''Change the currently selected device.'''
-        self.__entry_device.set_text(device_node)
+
+        # try to figure out device node if not specified:
+        receiver = self.selected_receiver
+
+        if receiver:
+            if device_node is None:
+                device_node = receiver.device
+            if device_node is None:
+                device_node = self.__hardware_manager.find_instance(receiver)
+
+        # try to select choosen device node from combo box:
+        tree_model = device_node and self.__combo_device.get_model()
+        tree_iter = tree_model and tree_model.get_iter_first() or None
+
+        while tree_iter is not None:
+            node, = tree_model.get(tree_iter, 1)
+
+            if node == device_node:
+                self.__combo_device.set_active_iter(tree_iter)
+                return
+
+            tree_iter = tree_model.iter_next(tree_iter)
+
+        # device node not found in combo box, fallback to modify its entry:
+        markup = (
+            receiver and not device_node and
+            _('<b>Warning:</b> Cannot find such receiver.') or '')
+
+        self.__entry_device.set_text(device_node or '')
+        self.__label_device_name.set_markup('<small>%s</small>' % markup)
 
     def __get_supplied_remote(self):
         '''Retrieve the supplied remot of the currently selected remote.'''
